import { useState } from "react";
import {
  UseFormSetError,
  UseFormClearErrors,
  UseFormSetValue,
} from "react-hook-form";

export const ALLOWED_IMAGE_TYPES = [
  "image/jpeg",
  "image/png",
  "image/gif",
  "image/webp",
];

export const ALLOWED_VIDEO_TYPES = ["video/mp4", "video/webm", "video/ogg"];

export const ALLOWED_PANORAMA_TYPES = [
  "image/jpeg",
  "image/jpg",
  "image/png",
  "image/gif",
  "image/webp",
];

export const ALLOWED_ICON_TYPES = [
  "image/svg+xml",
  "image/png",
  "image/jpg",
  "image/jpeg",
  "image/webp",
];

export const ALLOWED_WATERMARK_TYPES = [
  "image/svg+xml",
  "image/png",
  "image/jpg",
  "image/jpeg",
  "image/webp",
];

export const ALLOWED_3D_MODEL_TYPES = [
  ".glb",
  "glb",
  ".gltf",
  "gltf",
  "model/gltf-binary",
  ".vnd.ms-3d",
];

export const validateFileType = (file: File, mediaType: number) => {
  if (mediaType === 1 && !ALLOWED_IMAGE_TYPES.includes(file.type)) {
    return 'Для типа "Фото" разрешены только форматы: JPG, PNG, GIF, WEBP';
  }

  if (mediaType === 2 && !ALLOWED_VIDEO_TYPES.includes(file.type)) {
    return 'Для типа "Видео" разрешены только форматы: MP4, WEBM, OGG';
  }

  if (mediaType === 3 && !ALLOWED_ICON_TYPES.includes(file.type)) {
    return 'Для типа "Иконка" разрешены только форматы: SVG, PNG, JPG, JPEG, WEBP';
  }

  if (mediaType === 4 && !ALLOWED_WATERMARK_TYPES.includes(file.type)) {
    return 'Для типа "Водяной знак" разрешены только форматы: SVG, PNG, JPG, JPEG, WEBP';
  }

  if (mediaType === 5 && !ALLOWED_PANORAMA_TYPES.includes(file.type)) {
    return 'Для типа "Панорама" разрешены только форматы: JPG, PNG, GIF, WEBP';
  }

  if (mediaType === 6 && !ALLOWED_3D_MODEL_TYPES.includes(file.type)) {
    const extension = file.name.split(".").pop();
    const isMimeTypeValid = ["model/gltf-binary"].includes(file.type);
    const isExtensionValid =
      extension && ALLOWED_3D_MODEL_TYPES.includes(extension);
    if (!isMimeTypeValid && !isExtensionValid) {
      return 'Для типа "3D-модель" разрешены только форматы: GLB, GLTF';
    }
  }

  return null;
};

type UseMediaFileUploadProps = {
  selectedMediaType: number;
  setError: UseFormSetError<any>;
  clearErrors: UseFormClearErrors<any>;
  setValue: UseFormSetValue<any>;
};

export const useMediaFileUpload = ({
  selectedMediaType,
  setError,
  clearErrors,
  setValue,
}: UseMediaFileUploadProps) => {
  const [selectedFile, setSelectedFile] = useState<File | null>(null);
  const [previewUrl, setPreviewUrl] = useState<string | null>(null);

  const handleFileChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0];
    if (!file) return;

    if (selectedMediaType) {
      const error = validateFileType(file, selectedMediaType);
      if (error) {
        setError("file", { type: "manual", message: error });
        event.target.value = "";
        return;
      }
    }

    clearErrors("file");
    setValue("file", file);
    setSelectedFile(file);

    if (file.type.startsWith("image/")) {
      const url = URL.createObjectURL(file);
      setPreviewUrl(url);
    } else {
      setPreviewUrl(null);
    }
  };

  const handleMediaTypeChange = (newMediaType: number | null) => {
    setValue("media_type", newMediaType || null);

    if (selectedFile && newMediaType) {
      const error = validateFileType(selectedFile, newMediaType);
      if (error) {
        setError("file", { type: "manual", message: error });
        setValue("file", null);
        setSelectedFile(null);
        setPreviewUrl(null);
      } else {
        clearErrors("file");
      }
    }
  };

  return {
    selectedFile,
    setSelectedFile,
    previewUrl,
    setPreviewUrl,
    handleFileChange,
    handleMediaTypeChange,
  };
};
