import {
  Autocomplete,
  Box,
  TextField,
  FormControlLabel,
  Checkbox,
  Typography,
  Button,
} from "@mui/material";
import { Edit, useAutocomplete } from "@refinedev/mui";
import { useForm } from "@refinedev/react-hook-form";
import { Controller } from "react-hook-form";
import { useNavigate, useParams } from "react-router";
import { LinkedItems } from "../../components/LinkedItems";
import {
  StationItem,
  VehicleItem,
  stationFields,
  vehicleFields,
} from "./types";
import { useEffect, useState } from "react";
import { META_LANGUAGE, languageStore } from "@stores";
import { observer } from "mobx-react-lite";
import { LanguageSelector } from "@ui";

export const RouteEdit = observer(() => {
  const { language } = languageStore;
  const {
    saveButtonProps,
    register,
    control,
    formState: { errors },
    refineCore: { queryResult },
    setValue,
    getValues,
    watch,
  } = useForm({
    refineCoreProps: META_LANGUAGE(language),
  });
  const routeDirectionFromServer = watch("route_direction");

  const [routeDirection, setRouteDirection] = useState(false);
  const navigate = useNavigate();
  const directions = [
    {
      label: "Прямой",
      value: true,
    },
    {
      label: "Обратный",
      value: false,
    },
  ];

  const { id: routeId } = useParams<{ id: string }>();

  useEffect(() => {
    if (queryResult?.data?.data && Array.isArray(queryResult.data.data.path)) {
      const formattedPath = queryResult.data.data.path
        .map((coords) => coords.join(" "))
        .join("\n");

      setValue("path", formattedPath);
    }
  }, [queryResult?.data?.data, setValue]);

  const { autocompleteProps: carrierAutocompleteProps } = useAutocomplete({
    resource: "carrier",
    onSearch: (value) => [
      {
        field: "short_name",
        operator: "contains",
        value,
      },
    ],
    ...META_LANGUAGE(language),
  });

  const { autocompleteProps: governorAppealAutocompleteProps } =
    useAutocomplete({
      resource: "article",

      onSearch: (value) => [
        {
          field: "heading",
          operator: "contains",
          value,
        },
        {
          field: "media_type",
          operator: "contains",
          value,
        },
      ],
      ...META_LANGUAGE(language),
    });

  useEffect(() => {
    if (routeDirectionFromServer) {
      setRouteDirection(routeDirectionFromServer);
    }
  }, [routeDirectionFromServer]);

  return (
    <Edit saveButtonProps={saveButtonProps}>
      <Box sx={{ display: "flex", flexDirection: "column", gap: 1 }}>
        <Box
          component="form"
          sx={{ display: "flex", flexDirection: "column" }}
          autoComplete="off"
        >
          <LanguageSelector />
          <Controller
            control={control}
            name="carrier_id"
            rules={{ required: "Это поле является обязательным" }}
            defaultValue={null}
            render={({ field }) => (
              <Autocomplete
                {...carrierAutocompleteProps}
                value={
                  carrierAutocompleteProps.options.find(
                    (option) => option.id === field.value
                  ) || null
                }
                onChange={(_, value) => {
                  field.onChange(value?.id || "");
                }}
                getOptionLabel={(item) => {
                  return item ? item.short_name : "";
                }}
                isOptionEqualToValue={(option, value) => {
                  return option.id === value?.id;
                }}
                filterOptions={(options, { inputValue }) => {
                  return options.filter((option) =>
                    option.short_name
                      .toLowerCase()
                      .includes(inputValue.toLowerCase())
                  );
                }}
                renderInput={(params) => (
                  <TextField
                    {...params}
                    label="Выберите перевозчика"
                    margin="normal"
                    variant="outlined"
                    error={!!errors.carrier_id}
                    helperText={(errors as any)?.carrier_id?.message}
                    required
                  />
                )}
              />
            )}
          />

          <TextField
            {...register("route_number", {
              required: "Это поле является обязательным",
              setValueAs: (value) => String(value),
            })}
            error={!!(errors as any)?.route_number}
            helperText={(errors as any)?.route_number?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="text"
            label={"Номер маршрута"}
            name="route_number"
          />

          <input type="hidden" {...register("route_direction")} />

          <Autocomplete
            options={directions}
            value={directions.find((el) => el.value == routeDirection)}
            onChange={(_, element) => {
              if (element) {
                setValue("route_direction", element.value);
                setRouteDirection(element.value);
              }
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Прямой/обратный маршрут"
                margin="normal"
                variant="outlined"
                error={!!errors.arms}
                helperText={(errors as any)?.arms?.message}
                required
              />
            )}
          />

          <TextField
            {...register("path", {
              required: "Это поле является обязательным",
              setValueAs: (value: string) => {
                try {
                  const lines = value.trim().split("\n");
                  return lines.map((line) => {
                    const [lat, lon] = line
                      .trim()
                      .split(/[\s,]+/)
                      .map(Number);
                    return [lat, lon];
                  });
                } catch {
                  return [];
                }
              },
              validate: (value: unknown) => {
                if (!Array.isArray(value)) return "Неверный формат";
                if (value.length === 0)
                  return "Введите хотя бы одну пару координат";
                if (
                  !value.every(
                    (point: unknown) =>
                      Array.isArray(point) && point.length === 2
                  )
                ) {
                  return "Каждая строка должна содержать две координаты";
                }
                if (
                  !value.every((point: unknown[]) =>
                    point.every(
                      (coord: unknown) =>
                        !isNaN(Number(coord)) && typeof coord === "number"
                    )
                  )
                ) {
                  return "Координаты должны быть числами";
                }
                return true;
              },
            })}
            error={!!(errors as any)?.path}
            helperText={(errors as any)?.path?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="text"
            label={"Координаты маршрута *"}
            name="path"
            placeholder="55.7558 37.6173
55.7539 37.6208"
            multiline
            rows={4}
            sx={{
              marginBottom: 2,
            }}
          />

          <TextField
            {...register("route_sys_number", {
              required: "Это поле является обязательным",
            })}
            error={!!(errors as any)?.route_sys_number}
            helperText={(errors as any)?.route_sys_number?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="number"
            label={"Номер маршрута в Говорящем Городе *"}
            name="route_sys_number"
          />

          <Controller
            control={control}
            name="governor_appeal"
            defaultValue={null}
            render={({ field }) => (
              <Autocomplete
                {...governorAppealAutocompleteProps}
                value={
                  governorAppealAutocompleteProps.options.find(
                    (option) => option.id === field.value
                  ) ?? null
                }
                onChange={(_, value) => {
                  field.onChange(value?.id ?? "");
                }}
                getOptionLabel={(item) => {
                  return item ? item.heading : "";
                }}
                isOptionEqualToValue={(option, value) => {
                  return option.id === value?.id;
                }}
                filterOptions={(options, { inputValue }) => {
                  return options.filter((option) =>
                    option.heading
                      .toLowerCase()
                      .includes(inputValue.toLowerCase())
                  );
                }}
                renderInput={(params) => (
                  <TextField
                    {...params}
                    label="Обращение губернатора"
                    margin="normal"
                    variant="outlined"
                    error={!!errors.arms}
                    helperText={(errors as any)?.arms?.message}
                    required
                  />
                )}
              />
            )}
          />

          <TextField
            {...register("scale_min", {
              // required: 'Это поле является обязательным',
              setValueAs: (value) => Number(value),
            })}
            error={!!(errors as any)?.scale_min}
            helperText={(errors as any)?.scale_min?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="number"
            label={"Масштаб (мин)"}
            name="scale_min"
          />

          <TextField
            {...register("scale_max", {
              // required: 'Это поле является обязательным',
              setValueAs: (value) => Number(value),
            })}
            error={!!(errors as any)?.scale_max}
            helperText={(errors as any)?.scale_max?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="number"
            label={"Масштаб (макс)"}
            name="scale_max"
          />

          <TextField
            {...register("rotate", {
              // required: 'Это поле является обязательным',
              setValueAs: (value) => Number(value),
            })}
            error={!!(errors as any)?.rotate}
            helperText={(errors as any)?.rotate?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="number"
            label={"Поворот"}
            name="rotate"
          />

          <TextField
            {...register("center_latitude", {
              // required: 'Это поле является обязательным',
              setValueAs: (value) => Number(value),
            })}
            error={!!(errors as any)?.center_latitude}
            helperText={(errors as any)?.center_latitude?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="number"
            label={"Центр. широта"}
            name="center_latitude"
          />

          <TextField
            {...register("center_longitude", {
              // required: 'Это поле является обязательным',
              setValueAs: (value) => Number(value),
            })}
            error={!!(errors as any)?.center_longitude}
            helperText={(errors as any)?.center_longitude?.message}
            margin="normal"
            fullWidth
            slotProps={{ inputLabel: { shrink: true } }}
            type="number"
            label={"Центр. долгота"}
            name="center_longitude"
          />
        </Box>

        {routeId && (
          <>
            <LinkedItems<StationItem>
              type="edit"
              parentId={routeId}
              parentResource="route"
              childResource="station"
              fields={stationFields}
              title="станции"
              dragAllowed={true}
            />

            <LinkedItems<VehicleItem>
              type="edit"
              parentId={routeId}
              parentResource="route"
              childResource="vehicle"
              fields={vehicleFields}
              title="транспортные средства"
            />
          </>
        )}

        <Box sx={{ display: "flex", justifyContent: "flex-start" }}>
          <Button
            variant="contained"
            color="primary"
            onClick={() => navigate(`/route-preview/${routeId}`)}
          >
            Предпросмотр маршрута
          </Button>
        </Box>
      </Box>
    </Edit>
  );
});
